<?php

namespace Drupal\knowledge\Controller;

use Drupal\Component\Utility\Xss;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Link;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\Core\Utility\TableSort;
use Drupal\knowledge\Entity\KnowledgeCompetencyInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Class KnowledgeCompetencyController.
 *
 *  Returns responses for Competency routes.
 */
class KnowledgeCompetencyController extends ControllerBase implements ContainerInjectionInterface {

  /**
   * The date formatter.
   *
   * @var \Drupal\Core\Datetime\DateFormatter
   */
  protected $dateFormatter;

  /**
   * The renderer.
   *
   * @var \Drupal\Core\Render\Renderer
   */
  protected $renderer;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->dateFormatter = $container->get('date.formatter');
    $instance->renderer = $container->get('renderer');
    return $instance;
  }

  /**
   * Displays a Competency revision.
   *
   * @param int $knowledge_competency_revision
   *   The Competency revision ID.
   *
   * @return array
   *   An array suitable for drupal_render().
   */
  public function revisionShow($knowledge_competency_revision) {
    /** @var \Drupal\knowledge\KnowledgeCompetencyStorageInterface $competency_storage */
    $competency_storage = $this->entityTypeManager()->getStorage('knowledge_competency');
    /** @var \Drupal\knowledge\Entity\KnowledgeCompetencyInterface $knowledge_competency */
    $knowledge_competency = $competency_storage->loadRevision($knowledge_competency_revision);
    $view_builder = $this->entityTypeManager()->getViewBuilder('knowledge_competency');

    return $view_builder->view($knowledge_competency);
  }

  /**
   * Page title callback for a Competency revision.
   *
   * @param int $knowledge_competency_revision
   *   The Competency revision ID.
   *
   * @return string
   *   The page title.
   */
  public function revisionPageTitle($knowledge_competency_revision) {
    /** @var \Drupal\knowledge\KnowledgeCompetencyStorageInterface $competency_storage */
    $competency_storage = $this->entityTypeManager()->getStorage('knowledge_competency');
    /** @var \Drupal\knowledge\Entity\KnowledgeCompetencyInterface $knowledge_competency */
    $knowledge_competency = $competency_storage->loadRevision($knowledge_competency_revision);

    return $this->t('Revision of %title from %date', [
      '%title' => $knowledge_competency->label(),
      '%date' => $this->dateFormatter->format($knowledge_competency->getRevisionCreationTime()),
    ]);
  }

  /**
   * Generates an overview table of older revisions of a Competency.
   *
   * @param \Drupal\knowledge\Entity\KnowledgeCompetencyInterface $knowledge_competency
   *   A Competency object.
   *
   * @return array
   *   An array as expected by drupal_render().
   */
  public function revisionOverview(KnowledgeCompetencyInterface $knowledge_competency) {
    $account = $this->currentUser();
    /** @var \Drupal\knowledge\KnowledgeCompetencyStorageInterface $knowledge_competency_storage */
    $knowledge_competency_storage = $this->entityTypeManager()->getStorage('knowledge_competency');

    $build['#title'] = $this->t('Revisions for %title', ['%title' => $knowledge_competency->label()]);

    $header = [$this->t('Revision'), $this->t('Operations')];
    $revert_permission = (($account->hasPermission("revert all competency revisions") || $account->hasPermission('administer competency entities')));
    $delete_permission = (($account->hasPermission("delete all competency revisions") || $account->hasPermission('administer competency entities')));

    $rows = [];

    $vids = $knowledge_competency_storage->revisionIds($knowledge_competency);

    $latest_revision = TRUE;

    foreach (array_reverse($vids) as $vid) {
      /** @var \Drupal\knowledge\Entity\KnowledgeCompetencyInterface $revision */
      $revision = $knowledge_competency_storage->loadRevision($vid);
      $username = [
        '#theme' => 'username',
        '#account' => $revision->getRevisionUser(),
      ];

      // Use revision link to link to revisions that are not active.
      $date = $this->dateFormatter->format($revision->getRevisionCreationTime(), 'short');
      if ($vid != $knowledge_competency->getRevisionId()) {
        $link = Link::fromTextAndUrl($date, new Url('entity.knowledge_competency.revision', [
          'knowledge_competency' => $knowledge_competency->id(),
          'knowledge_competency_revision' => $vid,
        ]))->toString();
      }
      else {
        $link = $knowledge_competency->toLink($date)->toString();
      }

      $row = [];
      $column = [
        'data' => [
          '#type' => 'inline_template',
          '#template' => '{% trans %}{{ date }} by {{ username }}{% endtrans %}{% if message %}<p class="revision-log">{{ message }}</p>{% endif %}',
          '#context' => [
            'date' => $link,
            'username' => $this->renderer->renderInIsolation($username),
            'message' => [
              '#markup' => $revision->getRevisionLogMessage(),
              '#allowed_tags' => Xss::getHtmlTagList(),
            ],
          ],
        ],
      ];
      $row[] = $column;

      if ($latest_revision) {
        $row[] = [
          'data' => [
            '#prefix' => '<em>',
            '#markup' => $this->t('Current revision'),
            '#suffix' => '</em>',
          ],
        ];
        foreach ($row as &$current) {
          $current['class'] = ['revision-current'];
        }
        $latest_revision = FALSE;
      }
      else {
        $links = [];
        if ($revert_permission) {
          $links['revert'] = [
            'title' => $this->t('Revert'),
            'url' => Url::fromRoute('entity.knowledge_competency.revision_revert', [
              'knowledge_competency' => $knowledge_competency->id(),
              'knowledge_competency_revision' => $vid,
            ]),
          ];
        }

        if ($delete_permission) {
          $links['delete'] = [
            'title' => $this->t('Delete'),
            'url' => Url::fromRoute('entity.knowledge_competency.revision_delete', [
              'knowledge_competency' => $knowledge_competency->id(),
              'knowledge_competency_revision' => $vid,
            ]),
          ];
        }

        $row[] = [
          'data' => [
            '#type' => 'operations',
            '#links' => $links,
          ],
        ];
      }

      $rows[] = $row;
    }

    $build['knowledge_competency_revisions_table'] = [
      '#theme' => 'table',
      '#rows' => $rows,
      '#header' => $header,
    ];

    return $build;
  }

  /**
   * Returns the competency the belongs to the user.
   */
  public function userCompetency($user) {
    $account = $this->currentUser();
    $view_builder = $this->entityTypeManager()->getViewBuilder('knowledge_competency');
    $form_builder = $this->entityFormBuilder();
    $competency = $this->getCompetency($user);

    if ($account->hasPermission('edit competency entities')) {
      return $form_builder->getForm($competency);
    }

    return $view_builder->view($competency);
  }

  /**
   * Checks access for a specific request.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   Run access checks for this account (leader).
   * @param int $user
   *   The user the competency belongs to.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function accessCompetencyApproval(AccountInterface $account, int $user) {

    $account_id = $account->id();
    if ($account_id == 1) {
      return AccessResult::allowed();
    }
    $competency = $this->getCompetency($user);
    if (!$competency->isPendingContributor() && !$competency->isPendingPublisher()) {
      throw new NotFoundHttpException();
    }

    $user = $this->entityTypeManager()
      ->getStorage('user')
      ->load($user);
    $leader_id = $user->knowledge_leader->target_id;

    return AccessResult::allowedIf($account_id == $leader_id);
  }

  /**
   * The table report of competencies.
   */
  public function competencySummaryReport(Request $request) {

    $groups = field_group_info_groups('knowledge_competency', 'knowledge_competency', 'form', 'default');
    $result = views_get_view_result('competency_progress');
    $result = $result[0];
    $entity = $result->_entity;
    $total = $result->id;
    $rows = [];
    $order = 0;
    $candidate = 'Paddler';
    $contributor = 'Rider';
    $publisher = 'Pro';
    $i = 0;
    foreach ($groups['group_candidate']->children as $field) {
      $competency = $entity->$field->getFieldDefinition()->getLabel();
      $key = 'knowledge_competency__' . $field . '_' . $field . '_value';
      $key = substr($key, 0, 60);
      $value = $result->$key;
      $percentage = (int) (100 * ($value / $total));
      $i += 1;
      $rows[] = [$i, $candidate, $competency, $percentage];
    }
    foreach ($groups['group_contributor']->children as $field) {
      $competency = $entity->$field->getFieldDefinition()->getLabel();
      $key = 'knowledge_competency__' . $field . '_' . $field . '_value';
      $key = substr($key, 0, 60);
      $value = $result->$key;
      $percentage = (int) (100 * ($value / $total));
      $i += 1;
      $rows[] = [$i, $contributor, $competency, $percentage];
    }
    foreach ($groups['group_publisher']->children as $field) {
      $competency = $entity->$field->getFieldDefinition()->getLabel();
      $key = 'knowledge_competency__' . $field . '_' . $field . '_value';
      $key = substr($key, 0, 60);
      $value = $result->$key;
      $percentage = (int) (100 * ($value / $total));
      $i += 1;
      $rows[] = [$i, $publisher, $competency, $percentage];
    }
    $header = [
      'id' => [
        'data' => $this->t('Id'),
        'field' => 'id',
        'class' => [RESPONSIVE_PRIORITY_LOW],
        'sort' => 'asc',
      ],
      'role' => [
        'data' => $this->t('Role'),
        'field' => 'role',
      ],
      'competency' => [
        'data' => $this->t('Competency'),
        'field' => 'competency',
      ],
      'progress' => [
        'data' => $this->t('Percentage'),
        'field' => 'progress',
      ],
    ];

    $sort = TableSort::getSort($header, $request);
    $order_by = TableSort::getOrder($header, $request)['sql'];
    $desc = ($sort === 'desc');
    switch ($order_by) {
      case 'id':
        $order = 0;
        break;

      case 'role':
        $order = 1;
        break;

      case 'competency':
        $order = 2;
        break;

      case 'progress':
        $order = 3;
        break;
    }

    usort($rows, $this->tableSorter($order, $desc));
    $data = [];
    $colors = [];
    $colors['Paddler'] = 'rgba(54, 162, 235';
    $colors['Rider'] = 'rgba(54, 255, 200';
    $colors['Pro'] = 'rgba(255, 99, 132';
    foreach ($rows as $row) {
      $role = $row[1];
      $competency = $row[2];
      $percent = $row[3];

      $data['labels'][] = $competency;
      $data['datasets'][0]['data'][] = $percent;
      $data['datasets'][0]['backgroundColor'][] = $colors[$role] . ',.4)';
      $data['datasets'][0]['borderColor'][] = $colors[$role] . ',.9)';
    }

    $hash = 'hash';
    $build['graph_paddler'] = [
      '#type' => 'container',
      '#attributes' => [
        'class' => ['competency-report'],
        'id' => [$hash],
      ],
      '#attached' => [
        'library' => ['knowledge/competency_report'],
        'drupalSettings' => [
          'competency_report' => [
            $hash => $data,
          ],
        ],
      ],
    ];

    $build['table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#weight' => 100,
      '#rows' => $rows,
      '#attributes' => ['class' => ['competency-table-report']],
    ];
    $build['#attached']['library'][] = 'knowledge/competency_report';

    return $build;
  }

  /**
   * Custom data sort.
   */
  private function tableSorter($key, $desc) {
    return function ($a, $b) use ($key, $desc) {
      $res = strnatcmp($a[$key], $b[$key]);
      if ($desc) {
        $res = -1 * $res;
      }
      return $res;
    };
  }

  /**
   * Get the user's competency.
   */
  protected function getCompetency($user_id) {
    $competency = NULL;
    $storage = $this->entityTypeManager()
      ->getStorage('knowledge_competency');
    $query = $storage->getQuery();
    $result = $query
      ->condition('user_id', $user_id)
      ->accessCheck(FALSE)
      ->execute();

    if (count($result) == 1) {
      $competency = $storage->load(current($result));
    }
    if (count($result) == 0) {
      $competency = $storage->create([
        'user_id' => $user_id,
      ]);
    }

    return $competency;
  }

}
