<?php

namespace Drupal\knowledge\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityChangedInterface;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityConstraintViolationList;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\RevisionableInterface;
use Drupal\Core\Entity\RevisionLogEntityTrait;
use Drupal\Core\Entity\RevisionLogInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\user\UserInterface;
use Symfony\Component\Validator\ConstraintViolation;

/**
 * Defines the Competency entity.
 *
 * @ingroup knowledge
 *
 * @ContentEntityType(
 *   id = "knowledge_competency",
 *   label = @Translation("Competency"),
 *   handlers = {
 *     "storage" = "Drupal\knowledge\KnowledgeCompetencyStorage",
 *     "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *     "list_builder" = "Drupal\knowledge\KnowledgeCompetencyListBuilder",
 *     "views_data" = "Drupal\knowledge\Entity\KnowledgeCompetencyViewsData",
 *
 *     "form" = {
 *       "default" = "Drupal\knowledge\Form\KnowledgeCompetencyForm",
 *       "add" = "Drupal\knowledge\Form\KnowledgeCompetencyForm",
 *       "edit" = "Drupal\knowledge\Form\KnowledgeCompetencyForm",
 *       "delete" = "Drupal\knowledge\Form\KnowledgeCompetencyDeleteForm",
 *       "approve" = "Drupal\knowledge\Form\KnowledgeCompetencyApproveForm",
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\knowledge\KnowledgeCompetencyHtmlRouteProvider",
 *     },
 *     "access" = "Drupal\knowledge\KnowledgeCompetencyAccessControlHandler",
 *   },
 *   base_table = "knowledge_competency",
 *   revision_table = "knowledge_competency_revision",
 *   revision_data_table = "knowledge_competency_field_revision",
 *   show_revision_ui = TRUE,
 *   translatable = FALSE,
 *   admin_permission = "administer competency entities",
 *   entity_keys = {
 *     "id" = "id",
 *     "revision" = "vid",
 *     "uuid" = "uuid",
 *     "uid" = "user_id",
 *     "langcode" = "langcode",
 *   },
 *   revision_metadata_keys = {
 *     "revision_user" = "revision_uid",
 *     "revision_created" = "revision_timestamp",
 *     "revision_log_message" = "revision_log"
 *   },
 *   links = {
 *     "canonical" = "/admin/content/knowledge/competency/{knowledge_competency}",
 *     "add-form" = "/admin/content/knowledge/competency/add",
 *     "edit-form" = "/admin/content/knowledge/competency/{knowledge_competency}/edit",
 *     "delete-form" = "/admin/content/knowledge/competency/{knowledge_competency}/delete",
 *     "version-history" = "/admin/content/knowledge/competency/{knowledge_competency}/revisions",
 *     "revision" = "/admin/content/knowledge/competency/{knowledge_competency}/revisions/{knowledge_competency_revision}/view",
 *     "revision_revert" = "/admin/content/knowledge/competency/{knowledge_competency}/revisions/{knowledge_competency_revision}/revert",
 *     "revision_delete" = "/admin/content/knowledge/competency/{knowledge_competency}/revisions/{knowledge_competency_revision}/delete",
 *     "collection" = "/admin/content/knowledge/competency",
 *   },
 *   field_ui_base_route = "knowledge_competency.settings"
 * )
 */
class KnowledgeCompetency extends ContentEntityBase implements EntityChangedInterface, RevisionLogInterface, KnowledgeCompetencyInterface {

  use EntityChangedTrait;
  use RevisionLogEntityTrait;

  /**
   * {@inheritdoc}
   */
  protected function urlRouteParameters($rel) {
    $uri_route_parameters = parent::urlRouteParameters($rel);

    if ($rel === 'revision_revert' && $this instanceof RevisionableInterface) {
      $uri_route_parameters[$this->getEntityTypeId() . '_revision'] = $this->getRevisionId();
    }
    elseif ($rel === 'revision_delete' && $this instanceof RevisionableInterface) {
      $uri_route_parameters[$this->getEntityTypeId() . '_revision'] = $this->getRevisionId();
    }

    return $uri_route_parameters;
  }

  /**
   * {@inheritdoc}
   */
  public function preSave(EntityStorageInterface $storage) {
    parent::preSave($storage);
    $this->setNewRevision();
    // If no revision author has been set explicitly,
    // make the knowledge_competency owner the revision author.
    if (!$this->getRevisionUser()) {
      $this->setRevisionUserId($this->getOwnerId());
    }

    $groups = field_group_info_groups('knowledge_competency', 'knowledge_competency', 'form', 'default');
    $candidate_correct = 0;
    $candidate_total = 0;
    foreach ($groups['group_candidate']->children as $field) {
      $candidate_total += 1;
      if ($this->{$field}->value) {
        $candidate_correct += 1;
      }
    }
    $contributor_correct = 0;
    $contributor_total = 0;
    foreach ($groups['group_contributor']->children as $field) {
      $contributor_total += 1;
      if ($this->{$field}->value) {
        $contributor_correct += 1;
      }
    }
    $publisher_correct = 0;
    $publisher_total = 0;
    foreach ($groups['group_publisher']->children as $field) {
      $publisher_total += 1;
      if ($this->{$field}->value) {
        $publisher_correct += 1;
      }
    }

    $this->set('candidate_correct', $candidate_correct);
    $this->set('candidate_total', $candidate_total);
    $this->set('contributor_correct', $contributor_correct);
    $this->set('contributor_total', $contributor_total);
    $this->set('publisher_correct', $publisher_correct);
    $this->set('publisher_total', $publisher_total);

    $correct = $candidate_correct + $contributor_correct + $publisher_correct;
    $total = $candidate_total + $contributor_total + $publisher_total;
    $this->set('correct', $correct);
    $this->set('total', $total);
  }

  /**
   * {@inheritdoc}
   */
  public function getCreatedTime() {
    return $this->get('created')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCreatedTime($timestamp) {
    $this->set('created', $timestamp);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwner() {
    return $this->get('user_id')->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwnerId() {
    return $this->get('user_id')->target_id;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwnerId($uid) {
    $this->set('user_id', $uid);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwner(UserInterface $account) {
    $this->set('user_id', $account->id());
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function isPublished() {
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);
    // Add the published field.
    $fields += static::revisionLogBaseFieldDefinitions($entity_type);
    /** @var \Drupal\Core\Field\BaseFieldDefinition $fields['revision_log'] */
    $fields['revision_log']
      ->setDisplayOptions('form', ['region' => 'hidden'])
      ->setDisplayConfigurable('form', FALSE);

    $fields['user_id'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Learner'))
      ->setDescription(t('The user ID of Competency entity.'))
      ->setRevisionable(FALSE)
      ->setReadOnly(TRUE)
      ->setSetting('target_type', 'user')
      ->setSetting('handler', 'default')
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'author',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'weight' => 5,
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => '60',
          'autocomplete_type' => 'tags',
          'placeholder' => '',
        ],
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['contributor_coach'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Coach'))
      ->setDescription(t('The user who suggests promotion to contributor.'))
      ->setRevisionable(FALSE)
      ->setSetting('target_type', 'user')
      ->setSetting('handler', 'default')
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'author',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'weight' => 5,
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => '60',
          'autocomplete_type' => 'tags',
          'placeholder' => '',
        ],
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['contributor_leader'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Leader'))
      ->setDescription(t('The user who promotes the user to contributor.'))
      ->setRevisionable(FALSE)
      ->setSetting('target_type', 'user')
      ->setSetting('handler', 'default')
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'author',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'weight' => 5,
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => '60',
          'autocomplete_type' => 'tags',
          'placeholder' => '',
        ],
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['publisher_coach'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Coach'))
      ->setDescription(t('The user ID of Competency entity.'))
      ->setRevisionable(FALSE)
      ->setSetting('target_type', 'user')
      ->setSetting('handler', 'default')
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'author',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'weight' => 5,
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => '60',
          'autocomplete_type' => 'tags',
          'placeholder' => '',
        ],
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['publisher_leader'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Leader'))
      ->setDescription(t('The user who approves the promotion.'))
      ->setRevisionable(FALSE)
      ->setSetting('target_type', 'user')
      ->setSetting('handler', 'default')
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'author',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'weight' => 5,
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => '60',
          'autocomplete_type' => 'tags',
          'placeholder' => '',
        ],
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['contributor_proposed'] = BaseFieldDefinition::create('timestamp')
      ->setLabel(t('Proposed'))
      ->setDescription(t('The time that the entity was created.'))
      ->setDisplayConfigurable('form', FALSE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['contributor_approved'] = BaseFieldDefinition::create('timestamp')
      ->setLabel(t('Approved'))
      ->setDescription(t('The time that the entity was created.'))
      ->setDisplayConfigurable('form', FALSE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['publisher_proposed'] = BaseFieldDefinition::create('timestamp')
      ->setLabel(t('Proposed'))
      ->setDescription(t('The time that the entity was created.'))
      ->setDisplayConfigurable('form', FALSE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['publisher_approved'] = BaseFieldDefinition::create('timestamp')
      ->setLabel(t('Approved'))
      ->setDescription(t('The time that the entity was created.'))
      ->setDisplayConfigurable('form', FALSE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['candidate_correct'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Correct candidate competencies'))
      ->setDescription(t('The number of correct candidate items.'))
      ->setRequired(TRUE)
      ->setRevisionable(TRUE);

    $fields['candidate_total'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Total candidate competencies'))
      ->setDescription(t('The total candidate items.'))
      ->setRequired(TRUE)
      ->setRevisionable(TRUE);

    $fields['contributor_correct'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Correct contributor competencies'))
      ->setDescription(t('The number of correct contributor items.'))
      ->setRequired(TRUE)
      ->setRevisionable(TRUE);

    $fields['contributor_total'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Total contributor competencies'))
      ->setDescription(t('The total contributor items.'))
      ->setRequired(TRUE)
      ->setRevisionable(TRUE);

    $fields['publisher_correct'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Correct publisher competencies'))
      ->setDescription(t('The number of correct publisher items.'))
      ->setRequired(TRUE)
      ->setRevisionable(TRUE);

    $fields['publisher_total'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Total publisher competencies'))
      ->setDescription(t('The total publisher items.'))
      ->setRequired(TRUE)
      ->setRevisionable(TRUE);

    $fields['correct'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Correct competencies'))
      ->setDescription(t('The number of correct items.'))
      ->setRequired(TRUE)
      ->setRevisionable(TRUE);

    $fields['total'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Total competencies'))
      ->setDescription(t('The total competencies items.'))
      ->setRequired(TRUE)
      ->setRevisionable(TRUE);

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Created'))
      ->setDescription(t('The time that the entity was created.'));

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Changed'))
      ->setDescription(t('The time that the entity was last edited.'));

    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public function validate() {

    $query = $this->entityTypeManager()
      ->getStorage('knowledge_competency')
      ->getQuery();
    $query->condition('user_id', $this->getOwnerId());
    if ($this->id()) {
      $query->condition('id', $this->id(), '<>');
    }

    $result = $query->accessCheck(FALSE)->execute();

    $this->validated = TRUE;
    $violations = $this
      ->getTypedData()
      ->validate();
    if (count($result)) {
      $violations[] = new ConstraintViolation('Competency for user already exists', '', [], '', 'user_id', 'duplicate');
    }

    return new EntityConstraintViolationList($this, iterator_to_array($violations));
  }

  /**
   * {@inheritdoc}
   */
  public function isPendingContributor(): bool {
    return !$this->isPendingPublisher() && $this->contributor_coach->target_id != NULL && $this->contributor_leader->target_id == NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function isPendingPublisher(): bool {
    return $this->publisher_coach->target_id != NULL && $this->publisher_leader->target_id == NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function isPending(): bool {
    return $this->isPendingContributor() || $this->isPendingPublisher();
  }

}
