<?php

namespace Drupal\knowledge\Form;

use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\user\Entity\Role;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides the knowledge delete confirmation form.
 *
 * @internal
 */
class KnowledgeCompetencyApproveForm extends ConfirmFormBase {

  /**
   * The knowledge competency.
   *
   * @var \Drupal\knowledge\Entity\KnowledgeCompetencyInterface
   */
  protected $competency;

  /**
   * The user to be promoted.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $learner;

  /**
   * The role to ne promoted to.
   *
   * @var \Drupal\user\RoleInterface
   */
  protected $role;

  /**
   * The current user account.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $account;

  /**
   * The time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * The competency storage service.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $competencyStorage;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    // Instantiates this form class.
    $instance = parent::create($container);
    $instance->account = $container->get('current_user');
    $instance->time = $container->get('datetime.time');
    $instance->competencyStorage = $container->get('entity_type.manager')
      ->getStorage('knowledge_competency');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() : string {
    return "confirm_knowledge_competency_approve_form";
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $user = '') {
    $competency = $this->getCompetency($user);
    $this->setCompetency($competency);
    $form = parent::buildForm($form, $form_state);

    return $form;
  }

  /**
   * Get the user's competency.
   */
  protected function getCompetency($user) {
    $query = $this->competencyStorage->getQuery();
    $result = $query
      ->condition('user_id', $user)
      ->accessCheck(FALSE)
      ->execute();
    $competency = NULL;
    if (count($result) == 1) {
      $competency = $this->competencyStorage->load(current($result));
    }
    if (count($result) == 0) {
      $competency = $this->competencyStorage->create([
        'user_id' => $user,
      ]);
    }

    return $competency;
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText() {
    return $this->t('Promote');
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    // Point to the entity of which this knowledge is a reply.
    return $this->competency->toUrl();
  }

  /**
   * {@inheritdoc}
   */
  protected function getRedirectUrl() {
    return $this->getCancelUrl();
  }

  /**
   * {@inheritdoc}
   */
  public function getDescription() {
    return $this->t('Do you want to promote the user?');
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    $username = $this->learner ? $this->learner->label() : '<unknown>';
    $role = $this->role ? $this->role->label() : '<unknown>';
    $p = [
      '@username' => $username,
      '@role' => $role,
    ];
    return $this->t('Do you want to promote @username to a @role?', $p);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $competency = $this->competency;
    $learner = $this->learner;
    $role = $this->role->id();

    if ($learner->hasRole('knowledge_candidate')) {
      $learner->removeRole('knowledge_candidate');
    }
    if ($role == 'knowledge_publisher' && $learner->hasRole('knowledge_contributor')) {
      $learner->removeRole('knowledge_contributor');
    }

    $learner->addRole($role);
    $learner->save();
    if ($competency->isPendingPublisher()) {
      $competency->publisher_approved->value = $this->time->getCurrentTime();
      $competency->publisher_leader->target_id = $this->account->id();
    }
    elseif ($competency->isPendingContributor()) {
      $competency->contributor_approved->value = $this->time->getCurrentTime();
      $competency->contributor_leader->target_id = $this->account->id();
    }
    $competency->save();

    $form_state->setRedirect('entity.knowledge_competency.canonical', ['knowledge_competency' => $this->competency->id()]);
  }

  /**
   * Sets the competency pending approval.
   */
  public function setCompetency($competency) {
    $this->competency = $competency;
    $this->learner = $competency->getOwner();
    if ($competency->isPendingContributor()) {
      $this->role = Role::load('knowledge_contributor');
    }
    elseif ($competency->isPendingPublisher()) {
      $this->role = Role::load('knowledge_publisher');
    }

  }

}
