<?php

namespace Drupal\knowledge\Form;

use Drupal\Core\Entity\ContentEntityForm;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form controller for Competency edit forms.
 *
 * @ingroup knowledge
 */
class KnowledgeCompetencyForm extends ContentEntityForm {

  /**
   * The current user account.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $account;

  /**
   * The knowledge settings config.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $config;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatter
   */
  protected $dateFormatter;

  /**
   * The user role storage service.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $roleStorage;

  /**
   * The field group info array.
   *
   * @var array
   */
  protected $groups;

  /**
   * The Renderer service.
   *
   * @var \Drupal\Core\Render\Renderer
   */
  protected $render;

  /**
   * The block manager service.
   *
   * @var \Drupal\Core\Block\BlockManagerInterface
   */
  protected $blockManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    // Instantiates this form class.
    $instance = parent::create($container);
    $instance->account = $container->get('current_user');
    $instance->roleStorage = $container->get('entity_type.manager')->getStorage('user_role');
    $instance->config = $container->get('config.factory')->get('knowledge.settings');
    $instance->dateFormatter = $container->get('date.formatter');
    $instance->time = $container->get('datetime.time');
    $instance->groups = field_group_info_groups('knowledge_competency', 'knowledge_competency', 'form', 'default');
    $instance->render = $container->get('renderer');
    $instance->blockManager = $container->get('plugin.manager.block');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);
    // In several places within this function, we vary $form on:
    // - The current user's permissions.
    // - Whether the current user is authenticated or anonymous.
    // - The 'user.settings' configuration.
    // - The knowledge field's definition.
    $form['#cache']['contexts'][] = 'user';
    $form['#cache']['contexts'][] = 'user.roles:authenticated';

    unset($form['revision_information']);
    unset($form['revision']);
    /** @var \Drupal\knowledge\Entity\KnowledgeCompetencyInterface $competency */
    $competency = $this->entity;
    $user = $competency->getOwner();
    $roles = $user->getRoles();

    $is_publisher = in_array('knowledge_publisher', $roles);
    $is_contributor = in_array('knowledge_contributor', $roles) && !$is_publisher;
    $is_candidate = in_array('knowledge_candidate', $roles) && !$is_contributor && !$is_publisher;

    if (!($is_candidate || $is_contributor || $is_publisher)) {
      return ['#markup' => $this->t('@username is not a knowledge worker.', ['@username' => $user->toLink()->toString()])];
    }

    $form['user_id']['#type'] = 'hidden';

    $candidate_option = $this->config->get('competency.candidate');
    $has_candidate_option = $candidate_option != '_none';
    $contributor_option = $this->config->get('competency.contributor');
    $has_contributor_option = $contributor_option != '_none';
    $publisher_option = $this->config->get('competency.publisher');
    $has_publisher_option = $publisher_option != '_none';
    $all_option = $this->config->get('competency.all');
    $has_all_option = $all_option != '_none';

    if ($has_candidate_option) {
      if ($candidate_option == 'contributor' && $is_candidate) {
        $this->addPromoteContributorField($form);
        $this->addCandidateChecks($form, $candidate_option);
      }
      elseif ($candidate_option == 'publisher' && $is_contributor) {
        $this->addPromotePublisherField($form);
        $this->addCandidateChecks($form, $candidate_option);
      }
    }

    if ($has_contributor_option) {
      if ($contributor_option == 'contributor' && $is_candidate) {
        $this->addPromoteContributorField($form);
        $this->addContributorChecks($form, $contributor_option);
      }
      elseif ($contributor_option == 'publisher' && $is_contributor) {
        $this->addPromotePublisherField($form);
        $this->addContributorChecks($form, $contributor_option);
      }
    }

    if ($has_publisher_option) {
      if ($publisher_option == 'contributor' && $is_candidate) {
        $this->addPromoteContributorField($form);
        $this->addPublisherChecks($form, $publisher_option);
      }
      elseif ($publisher_option == 'publisher' && $is_contributor) {
        $this->addPromotePublisherField($form);
        $this->addPublisherChecks($form, $publisher_option);
      }
    }

    if ($has_all_option) {
      if ($all_option == 'contributor' && $is_candidate) {
        $this->addPromoteContributorField($form);
        $this->addAllChecks($form, $all_option);
      }
      elseif ($all_option == 'publisher' && $is_contributor) {
        $this->addPromotePublisherField($form);
        $this->addAllChecks($form, $all_option);
      }
    }

    $form['#attached']['library'][] = 'knowledge/competency_form';

    return $form;
  }

  /**
   * Renders a block plugin as markup.
   */
  protected function renderPluginBlock($plugin_id, $config = []) {
    $plugin_block = $this->blockManager->createInstance($plugin_id, $config);
    // Some blocks might implement access check.
    $access_result = $plugin_block->access($this->account);

    // Return empty render array if user doesn't have access.
    // $access_result can be boolean or an AccessResult class.
    if (is_object($access_result)
      && $access_result->isForbidden()
      || is_bool($access_result) && !$access_result) {
      return [];
    }

    $render = $plugin_block->build();

    $this->render->addCacheableDependency($render, $plugin_block);

    return $render;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $contributor_proposal = $form_state->getValue('contributor_proposal');
    $publisher_proposal = $form_state->getValue('publisher_proposal');
    /** @var \Drupal\knowledge\Entity\KnowledgeCompetencyInterface $competency */
    $competency = $this->entity;
    if (empty($competency->contributor_proposed->value) && $contributor_proposal) {
      $competency->contributor_proposed->value = $this->time->getCurrentTime();
      $competency->contributor_coach->target_id = $this->account->id();
    }
    elseif (!empty($competency->contributor_proposed->value) && !$contributor_proposal) {
      $competency->contributor_proposed->value = NULL;
      $competency->contributor_coach->target_id = NULL;
      $competency->contributor_leader->target_id = NULL;
    }
    if (empty($competency->publisher_proposed->value) && $publisher_proposal) {
      $competency->publisher_proposed->value = $this->time->getCurrentTime();
      $competency->publisher_coach->target_id = $this->account->id();
    }
    elseif (!empty($competency->publisher_proposed->value) && !$publisher_proposal) {
      $competency->publisher_proposed->value = NULL;
      $competency->publisher_coach->target_id = NULL;
      $competency->publisher_leader->target_id = NULL;
    }
    $status = parent::save($form, $form_state);

    switch ($status) {
      case SAVED_NEW:
        $this->messenger()->addMessage($this->t('Created the %label Competency.', [
          '%label' => $competency->label(),
        ]));
        break;

      default:
        $this->messenger()->addMessage($this->t('Saved the %label Competency.', [
          '%label' => $competency->label(),
        ]));
    }

    return $status;
  }

  /**
   * Adds the contributor proposal field.
   */
  protected function addPromoteContributorField(array &$form) {
    $contributor = $this->roleStorage->load('knowledge_contributor');
    /** @var \Drupal\knowledge\Entity\KnowledgeCompetencyInterface $competency */
    $competency = $this->entity;
    $is_proposed = !empty($competency->contributor_proposed->value);
    if ($is_proposed) {
      $contributor_coach = $competency->contributor_coach->entity->label();
      $date = $this->dateFormatter->format($competency->contributor_proposed->value, 'long');
      $description = $this->t('@username proposed on @date', [
        '@username' => $contributor_coach,
        '@date' => $date,
      ]);
    }
    else {
      $description = $this->t('Propose user be promoted to a @contributor.',
        [
          '@contributor' => $contributor->label(),
        ]);
    }
    $form['contributor_proposal'] = [
      '#type' => 'checkbox',
      '#title' => $contributor->label(),
      '#description' => $description,
      '#group' => 'second',
      '#default_value' => $is_proposed,
      '#weight' => 0,
    ];
  }

  /**
   * Adds the publisher proposal field.
   */
  protected function addPromotePublisherField(array &$form) {
    $publisher = $this->roleStorage->load('knowledge_publisher');
    /** @var \Drupal\knowledge\Entity\KnowledgeCompetencyInterface $competency */
    $competency = $this->entity;
    $is_proposed = !empty($competency->publisher_proposed->value);
    if ($is_proposed) {
      $publisher_coach = $competency->publisher_coach->entity->label();
      $date = $this->dateFormatter->format($competency->publisher_proposed->value, 'long');
      $description = $this->t('@username proposed on @date', [
        '@username' => $publisher_coach,
        '@date' => $date,
      ]);
    }
    else {
      $description = $this->t('Propose user be promoted to a @publisher.',
        [
          '@publisher' => $publisher->label(),
        ]);
    }
    $form['publisher_proposal'] = [
      '#type' => 'checkbox',
      '#title' => $publisher->label(),
      '#description' => $description,
      '#group' => 'second',
      '#default_value' => $is_proposed,
    ];

  }

  /**
   * Adds state checks to a field based on a role.
   */
  protected function addChecks(array &$form, string $role_name, string $group_name) {
    foreach ($this->groups[$group_name]->children as $field_name) {
      $form[$role_name . '_proposal']['#states']['visible'][':input[name="' . $field_name . '[value]"]'] = [
        'checked' => TRUE,
      ];
    }
  }

  /**
   * Adds candidate state checks to a field.
   */
  protected function addCandidateChecks(array &$form, string $role_name) {
    $this->addChecks($form, $role_name, 'group_candidate');
  }

  /**
   * Adds contributor state checks to a field.
   */
  protected function addContributorChecks(array &$form, string $role_name) {
    $this->addChecks($form, $role_name, 'group_contributor');
  }

  /**
   * Adds publisher state checks to a field.
   */
  protected function addPublisherChecks(array &$form, string $role_name) {
    $this->addChecks($form, $role_name, 'group_publisher');
  }

  /**
   * Adds all state checks to a field.
   */
  protected function addAllChecks(array &$form, string $role_name) {
    $this->addCandidateChecks($form, $role_name);
    $this->addContributorChecks($form, $role_name);
    $this->addPublisherChecks($form, $role_name);
  }

}
